import numpy
import os
import re


def findall_pat(pat, content, result="list"):
    """
    :param result:
    :param pat: string
    :param content: string
    :return: list
    """
    pat = re.compile(pat)
    find = re.findall(pat, content)
    if result == "list":
        return find
    elif result == "one":
        if len(find) > 0:
            return find[0]
        else:
            print("NOT FOUND")
            return find
    else:
        print("usage:\nresult='list'(default)or'one")


def auto_download(kegg_id):
    print("waiting for coding...")
    download_path = ""
    return download_path


def compute_pos1(g_pos_list):
    start_pos = int(g_pos_list[0][0])
    stop_pos = int(g_pos_list[0][1])
    middle_pos = (start_pos + stop_pos) / 2
    return middle_pos


def compute_pos2(g_pos_list):
    start_pos = int(g_pos_list[0][0])
    stop_pos = int(g_pos_list[0][1])
    for item in g_pos_list[1:]:
        stop_pos = int(item[1]) - (int(item[0]) - stop_pos)
    middle_pos = (start_pos + stop_pos) / 2
    return middle_pos


def sorted_gene(gene_pos):
    name_gene = []
    middle_gene = []
    for key in gene_pos:
        name_gene.append(key)
        g_pos_list = gene_pos[key]
        if len(g_pos_list) == 1:
            middle_pos = compute_pos1(g_pos_list)
        else:
            middle_pos = compute_pos2(g_pos_list)
        middle_gene.append(middle_pos)
    name_gene = [name_gene for (middle_gene, name_gene) in sorted(zip(middle_gene, name_gene))]
    return name_gene


def co_expression_strand(gene_sort, gene_strand):
    list_gene_strand = []
    for item in gene_sort:
        list_gene_strand.append(gene_strand[item])
    return list_gene_strand


def matrix_strand_to_int(matrix_strand):
    matrix_strand[matrix_strand == "++"] = 22
    matrix_strand[matrix_strand == "+-"] = 21
    matrix_strand[matrix_strand == "-+"] = 12
    matrix_strand[matrix_strand == "--"] = 11
    matrix_strand[matrix_strand == " +"] = 92
    matrix_strand[matrix_strand == " -"] = 91
    matrix_strand[matrix_strand == "+ "] = 29
    matrix_strand[matrix_strand == "- "] = 19
    matrix_strand = matrix_strand.astype("int")
    return matrix_strand


def matrix_to_list_gene_strand(list_gene_strand):
    list_a = list_gene_strand.copy()
    list_a.insert(0, " ")
    a = numpy.array([list_a]).T
    list_b = list_gene_strand.copy()
    list_b.append(" ")
    b = numpy.array([list_b]).T
    matrix_strand = numpy.char.add(a, b)
    matrix_strand_int = matrix_strand_to_int(matrix_strand)
    list_strand_int = matrix_strand_int.T.tolist()[0]
    return list_strand_int


def get_list_gene_strand(gene_sort, gene_strand_dict):
    list_gene_strand = co_expression_strand(gene_sort, gene_strand_dict)
    list_gene_strand = matrix_to_list_gene_strand(list_gene_strand)
    return list_gene_strand


def gene_pos_dict_remove_error(gene_pos_dict):
    if "-" in gene_pos_dict:
        gene_pos_dict.pop("-")
        return gene_pos_dict
    else:
        return gene_pos_dict


def gene_strand_dict_remove_error(gene_strand_dict):
    if "-" in gene_strand_dict:
        gene_strand_dict.pop("-")
        return gene_strand_dict
    else:
        return gene_strand_dict


def get_gene_pos_strand(simple_gff_file):
    simple_gff_fp = open(simple_gff_file, 'r')
    gene_pos_dict = {}
    gene_strand_dict = {}
    while True:
        line = simple_gff_fp.readline().strip()
        if not line:
            break
        gene_information = line.split("\t")
        # print(gene_information)
        g_b_id = gene_information[0]
        g_start = int(gene_information[4])
        g_stop = int(gene_information[5])
        g_strand = gene_information[-1]
        if g_b_id not in gene_pos_dict:
            gene_pos_dict[g_b_id] = [(g_start, g_stop)]
        else:
            gene_pos_dict[g_b_id].append((g_start, g_stop))
        gene_strand_dict[g_b_id] = g_strand
    gene_pos_dict = gene_pos_dict_remove_error(gene_pos_dict)
    gene_strand_dict = gene_strand_dict_remove_error(gene_strand_dict)
    return gene_pos_dict, gene_strand_dict


def query_strand(num):
    if num == 2:
        return "+"
    elif num == 1:
        return "-"
    elif num == 9:
        return " "
    else:
        return ""


def partition_gene(gene_sorted, list_gene_strand):
    # print(list_gene_strand)
    gene_start = list_gene_strand[0]
    gene1_strand = gene_start % 10
    gene_index_matrix = [[0]]
    gene_strand_matrix = [[query_strand(gene1_strand)]]
    i = 0
    gene_num = len(list_gene_strand)
    # print(gene_num)
    j = 1
    while j < gene_num - 1:
        gene_j_strand = list_gene_strand[j] % 10
        if list_gene_strand[j] == (3 - gene1_strand) * 11:
            tmp_index = gene_index_matrix[i].pop()
            gene_index_matrix.append([tmp_index, j])
            tmp_strand = gene_strand_matrix[i].pop()
            gene_strand_matrix.append([tmp_strand, query_strand(gene_j_strand)])
            k = 2
            while list_gene_strand[j - k] != gene1_strand * 11:
                tmp_index_pre = gene_index_matrix[i][1-k]
                tmp_index_pre_pre = gene_index_matrix[i][0-k]
                gene_index_matrix[i + 1].insert(0, tmp_index_pre)
                gene_index_matrix[i + 1].insert(0, tmp_index_pre_pre)
                tmp_strand_pre = gene_strand_matrix[i][1-k]
                tmp_strand_pre_pre = gene_strand_matrix[i][0-k]
                gene_strand_matrix[i + 1].insert(0, tmp_strand_pre)
                gene_strand_matrix[i + 1].insert(0, tmp_strand_pre_pre)
                k = k + 2
            gene1_strand = 3 - gene1_strand
            i = i + 1
        else:
            gene_index_matrix[i].append(j)
            gene_strand_matrix[i].append(query_strand(gene_j_strand))
        j = j + 1
    sorted_gene_matrix = []
    for each_ in gene_index_matrix:
        sorted_gene_matrix.append([])
        for item in each_:
            sorted_gene_matrix[-1].append(gene_sorted[item])
    # print(len(sorted_gene_matrix))
    # print(len(gene_index_matrix))
    # print(len(gene_strand_matrix))
    # i = 0
    # for each_ in gene_strand_matrix:
    #     frv = each_.count("+")
    #     rev = each_.count("-")
    #     if frv > rev:
    #         if rev != 0:
    #             if i+1 < len(gene_strand_matrix) and i-1 >= 0:
    #                 print(gene_strand_matrix[i - 1], each_, gene_strand_matrix[i + 1])
    #                 print(gene_index_matrix[i-1],gene_index_matrix[i],gene_index_matrix[i+1])
    #                 print(sorted_gene_matrix[i-1], sorted_gene_matrix[i],sorted_gene_matrix[i+1])
    #     else:
    #         if frv != 0:
    #             if i + 1 < len(gene_strand_matrix) and i - 1 >= 0:
    #                 print(gene_strand_matrix[i - 1], each_, gene_strand_matrix[i + 1])
    #                 print(gene_index_matrix[i - 1], gene_index_matrix[i], gene_index_matrix[i + 1])
    #                 print(sorted_gene_matrix[i - 1], sorted_gene_matrix[i], sorted_gene_matrix[i + 1])
    #     i = i + 1
    # print(gene_strand_matrix)
    # print(gene_index_matrix)
    # print(sorted_gene_matrix)
    return gene_strand_matrix, gene_index_matrix, sorted_gene_matrix


def from_simple_gff_information_to_get(gene_pos_dict, gene_strand_dict):
    gene_sort = sorted_gene(gene_pos_dict)
    list_strand = get_list_gene_strand(gene_sort, gene_strand_dict)
    final_gene_strand, final_gene_index, final_gene_sort = partition_gene(gene_sort, list_strand)
    return final_gene_strand, final_gene_index, final_gene_sort


def get_annotation(gff_file):
    gff_fp = open(gff_file, "r")
    genes = []
    while True:
        line = gff_fp.readline().strip()
        if not line:
            break
        line_content = line.split("\t")
        if len(line_content) > 1:
            start_pos = int(line_content[3])
            stop_pos = int(line_content[4])
            annotation_dict = {}
            annotation_ls = line_content[-1].split(";")
            for ann in annotation_ls:
                split_ann = ann.split("=")
                key = split_ann[0]
                info = split_ann[1].split(",")
                annotation_dict[key] = info
            genes.append([(start_pos, stop_pos), annotation_dict])
        else:
            # print(line)
            pass
    gff_fp.close()
    return genes


def filter_genes(gff_file):
    genes = get_annotation(gff_file)
    i = 0
    curr_len = len(genes)
    tmp_need_del = []
    while i < curr_len:
        annotation = genes[i][1]
        if 'Parent' in annotation:
            # if 'locus_tag' in annotation:
            #     print(genes[i])
            record = i - 1
            gene_id = annotation['Parent']
            while record > 0:
                if genes[record][-1]['ID'] == gene_id:
                    if 'part' not in genes[record][-1]:
                        if genes[record][0] != genes[i][0]:
                            genes[record].insert(-1, genes[i][0])
                            tmp_need_del.append(genes[i])
                            # if 'pseudo' not in genes[record][-1]:
                            #     print(genes[record])  # only 'prfB' without keyword 'pesudo'
                        else:
                            tmp_need_del.append(genes[i])
                        record = 0
                    else:
                        tmp_need_del.append(genes[i])
                        record = 0
                else:
                    record = record - 1
        i = i + 1
    for tmp_del in tmp_need_del:
        genes.remove(tmp_del)
    return genes


def get_strand_dict(gff_file):
    gff_fp = open(gff_file, "r")
    strand_id_dict = {}
    while True:
        line = gff_fp.readline().strip()
        if not line:
            break
        line_content = line.split("\t")
        if len(line_content) > 1:
            _id = findall_pat("ID\=([A-Za-z0-9]+)\;", line_content[-1], "one")
            strand_id_dict[_id] = line_content[6]
        else:
            pass
    gff_fp.close()
    return strand_id_dict


def generate_simple_gff(gff_file, output_path):
    specie_name = gff_file.split("/")
    specie_name = specie_name[-1]
    tmp_path = output_path + "tmp/"
    if not os.path.exists(tmp_path):
        os.system("mkdir " + tmp_path)
    simple_gff_path = tmp_path + "simple_" + specie_name
    genes = filter_genes(gff_file)
    simple_gff_fp = open(simple_gff_path, 'w')
    for gene in genes:
        if 'locus_tag' not in gene[-1]:
            pass
            # print(gene)  # always with one (start, stop)
            # if 'gbkey' not in gene[-1]:
            #     print(gene)  # NONE
            # start = gene[0][0]
            # stop = gene[0][1]
            # _id = gene[1]['ID'][0]
            # gb_key = gene[1]['gbkey'][0]
            # simple_gff_fp.write("-\t-\t-\t-\t" + str(start) + "\t" + str(stop) + "\t" + _id + "\t" + gb_key + "\n")
        else:
            if len(gene) == 2:
                start = gene[0][0]
                stop = gene[0][1]
                _id = gene[1]['ID'][0]
                gb_key = gene[1]['gbkey'][0]
                locus_tag = gene[1]['locus_tag'][0]
                symbol = gene[1]['gene'][0]
                synonym = ";".join(gene[1]['gene_synonym'])
                if 'part' in gene[1]:
                    part = gene[1]['part'][0]
                    part_i = part.split("/")[0]
                    for gene_syn in gene[-1]['gene_synonym']:
                        gene_syn = gene_syn + "_" + part_i
                        synonym = synonym + ";" + gene_syn
                    synonym = synonym + ";" + symbol + "_" + part_i
                    simple_gff_fp.write(locus_tag + "\t" + symbol + "\t" + synonym + "\t" + part + "\t"
                                        + str(start) + "\t" + str(stop) + "\t" + _id + "\t" + gb_key + "\n")
                else:
                    simple_gff_fp.write(locus_tag + "\t" + symbol + "\t" + synonym + "\t" + "-" + "\t"
                                        + str(start) + "\t" + str(stop) + "\t" + _id + "\t" + gb_key + "\n")
            else:
                # if 'pseudogene' not in gene[-1]['gene_biotype']:
                #     print(4, gene)  # prfB
                start = gene[0][0]
                stop = gene[0][1]
                _id = gene[-1]['ID'][0]
                gb_key = gene[-1]['gbkey'][0]
                locus_tag = gene[-1]['locus_tag'][0]
                symbol = gene[-1]['gene'][0]
                synonym = ";".join(gene[-1]['gene_synonym'])
                simple_gff_fp.write(locus_tag + "\t" + symbol + "\t" + synonym + "\t" + "-" + "\t"
                                    + str(start) + "\t" + str(stop) + "\t" + _id + "\t" + gb_key + "\n")
                j = 1
                while j < len(gene) - 1:
                    start = gene[j][0]
                    stop = gene[j][1]
                    _id = gene[-1]['ID'][0]
                    gb_key = gene[-1]['gbkey'][0]
                    locus_tag = gene[-1]['locus_tag'][0]
                    symbol = gene[-1]['gene'][0] + "_" + str(j)
                    synonym = ";".join(gene[-1]['gene_synonym'])
                    for gene_syn in gene[-1]['gene_synonym']:
                        gene_syn = gene_syn + "_" + str(j)
                        synonym = synonym + ";" + gene_syn
                    simple_gff_fp.write(locus_tag + "\t" + symbol + "\t" + synonym + "\t" + "-" + "\t"
                                        + str(start) + "\t" + str(stop) + "\t" + _id + "\t" + gb_key + "\n")
                    j = j + 1
    simple_gff_fp.close()
    strand_dict = get_strand_dict(gff_file)
    simple_fp = open(simple_gff_path, "r")
    write_fp = open(simple_gff_path + "_2", "w")
    while True:
        line = simple_fp.readline().strip()
        if not line:
            break
        line_content = line.split("\t")
        _id = line_content[6]
        if _id in strand_dict:
            write_fp.write(line + "\t" + strand_dict[_id] + "\n")
        else:
            print(_id)
            write_fp.write(line + "\t-\n")
    write_fp.close()
    simple_fp.close()
    os.system("rm -f " + simple_gff_path)
    os.system("mv " + simple_gff_path + "_2 " + simple_gff_path)
    return simple_gff_path


if __name__ == "__main__":
    test_gff_path = "/home/lyd/document/2018.1/gamma_domain/simple_eco.gff_3"
    test_gene_pos, test_gene_strand = get_gene_pos_strand(test_gff_path)
    _gene_strand, _gene_index, _gene_sort = \
        from_simple_gff_information_to_get(test_gene_pos, test_gene_strand)

    # test get_list_gene_strand()
    a = ["+", "-", "-"]
    d = matrix_to_list_gene_strand(a)
    print(d)



import numpy


def get_co_expression_by_condition(matrix_a):
    condition_num = matrix_a.shape[1]
    print("condition_num:", condition_num)
    for i in range(condition_num):
        c_i_expression = matrix_a[..., i:i + 1].copy()
        c_minus_i = matrix_a - c_i_expression
        if i == 0:
            matrix_a_compair = c_minus_i
        else:
            matrix_a_compair = numpy.column_stack((matrix_a_compair, c_minus_i))
    matrix_a_compair[matrix_a_compair >= 0] = 1
    matrix_a_compair[matrix_a_compair < 0] = -1
    return matrix_a_compair


def get_co_expression_gene_i_with_j(matrix_condition_s_v):
    gene_num = matrix_condition_s_v.shape[0]
    print("gene_num:", gene_num)
    matrix_c_i_j = numpy.zeros([gene_num, gene_num])
    for j in range(gene_num - 1):
        s_v_j_expression = matrix_condition_s_v[0:1, ...]
        s_v_other_expression = numpy.delete(matrix_condition_s_v, 0, axis=0)
        matrix_condition_s_v = s_v_other_expression
        # print(s_v_j_expression, "\n")
        # print(s_v_other_expression, "\n")
        s_v_multiply_j = s_v_other_expression * s_v_j_expression
        # print(s_v_multiply_j, "\n")
        matrix_c_i_j_iter = j + 1
        # print(s_v_multiply_j.sum(axis=1), "\n")
        matrix_c_i_j[matrix_c_i_j_iter:, j] = s_v_multiply_j.sum(axis=1)
        # print(matrix_c_i_j, "\n")
    return matrix_c_i_j


def compute_co_expression_by_c_i_j(matrix_a):
    matrix_condition_s_v = get_co_expression_by_condition(matrix_a)
    matrix_c_i_j = get_co_expression_gene_i_with_j(matrix_condition_s_v)
    condition_num = matrix_a.shape[1]
    matrix_c_i_j = matrix_c_i_j /(condition_num * condition_num)
    matrix_t = matrix_c_i_j.T
    matrix_c_i_j_final = matrix_c_i_j + matrix_t
    return matrix_c_i_j_final


if __name__ == "__main__":
    # test matrix_co_expression()
    c1 = [50, 33, 20, 19, 13]
    c2 = [51, 34, 21, 18, 12]
    c3 = [52, 35, 22, 17, 12]
    matrix_test = numpy.array([c1, c2, c3]).T
    print(compute_co_expression_by_c_i_j(matrix_test))



import numpy
import sys


class Node:
    def __init__(self, l, r):
        self.a = 0
        self.left_child = l
        self.right_child = r
        self.list_node = []

    def set_left(self, _node):
        self.left_child = _node

    def set_right(self, _node):
        self.right_child = _node

    def to_list(self, _node):
        if type(_node) is numpy.int64:
            # print(_node)
            self.list_node.append(_node)
        else:
            self.to_list(_node.left_child)
            self.to_list(_node.right_child)
        self.list_node = sorted(self.list_node)
        return self.list_node


class Gamma:
    def __init__(self, matrix):
        self.matrix = matrix
        self.trees = []
        self.index_trees = []

    def clustering2(self, t):
        cluster_matrix = self.matrix.copy()
        while cluster_matrix.max() >= t:
            # print(cluster_matrix)
            a = cluster_matrix.max()
            x_array, y_array = numpy.where(cluster_matrix == a)
            x = x_array[0]
            y = y_array[0]
            # print(x, y)
            new_node = Node(x, y)
            new_node.a = a
            left_exist = self.is_exist(new_node.left_child)
            right_exist = self.is_exist(new_node.right_child)
            if left_exist == 0 and right_exist == 0:
                self.save_to_trees(new_node)
            elif left_exist == 0 and right_exist == 1:
                tree_index = self.get_index(new_node.right_child)
                new_node.set_right(self.trees[tree_index])
                self.trees.remove(self.trees[tree_index])
                self.index_trees.remove(self.index_trees[tree_index])
                self.save_to_trees(new_node)
            elif left_exist == 1 and right_exist == 0:
                tree_index = self.get_index(new_node.left_child)
                new_node.set_left(self.trees[tree_index])
                self.trees.remove(self.trees[tree_index])
                self.index_trees.remove(self.index_trees[tree_index])
                self.save_to_trees(new_node)
            elif left_exist == 1 and right_exist == 1:
                tree_index = self.get_index(new_node.left_child)
                new_node.set_left(self.trees[tree_index])
                self.trees.remove(self.trees[tree_index])
                self.index_trees.remove(self.index_trees[tree_index])
                tree_index = self.get_index(new_node.right_child)
                new_node.set_right(self.trees[tree_index])
                self.trees.remove(self.trees[tree_index])
                self.index_trees.remove(self.index_trees[tree_index])
                self.save_to_trees(new_node)
            else:
                print("something wrong")
            cluster_matrix[x][y] = -2
            # print(cluster_matrix)
        for i in range(len(cluster_matrix)):
            if not self.is_exist(i):
                self.index_trees.append([i])
        return self.index_trees

    def clustering(self):
        cluster_matrix = self.matrix.copy()
        while cluster_matrix.max() > 0:
            a = cluster_matrix.max()
            x_array, y_array = numpy.where(cluster_matrix == a)
            x = x_array[0]
            y = y_array[0]
            new_node = Node(x, y)
            new_node.a = a
            left_exist = self.is_exist(new_node.left_child)
            right_exist = self.is_exist(new_node.right_child)
            if left_exist == 0 and right_exist == 0:
                self.save_to_trees(new_node)
            elif left_exist == 0 and right_exist == 1:
                tree_index = self.get_index(new_node.right_child)
                new_node.set_right(self.trees[tree_index])
                self.save_to_trees(new_node)
            elif left_exist == 1 and right_exist == 0:
                tree_index = self.get_index(new_node.left_child)
                new_node.set_left(self.trees[tree_index])
                self.save_to_trees(new_node)
            elif left_exist == 1 and right_exist == 1:
                tree_index = self.get_index(new_node.left_child)
                new_node.set_left(self.trees[tree_index])
                tree_index = self.get_index(new_node.right_child)
                new_node.set_right(self.trees[tree_index])
                self.save_to_trees(new_node)
            else:
                print("something wrong")
            cluster_matrix[x][y] = -2
            # print(cluster_matrix)

    def is_exist(self, num):
        for each in self.index_trees:
            for single in each:
                if num == single:
                    return 1
        return 0

    def get_index(self, num):
        index_list = []
        size_list = []
        for each in self.index_trees:
            # print(each)
            for single in each:
                if num == single:
                    index_list.append(self.index_trees.index(each))
                    size_list.append(len(each))
        return index_list[size_list.index(max(size_list))]

    def save_to_trees(self, _node):
        self.index_trees.append(_node.to_list(_node))
        self.trees.append(_node)

    def gamma_threshold(self, t):
        # negative_predict = []
        for _tree in self.trees:
            if _tree.a < t:
                self.index_trees.remove(_tree.list_node)
                # negative_predict.append(_tree.list_node)
        return self.index_trees

    def statistics_gamma(self):
        a_list = set()
        for _tree in self.trees:
            a_list.add(_tree.a)
        return sorted(list(a_list))


def add_name(_result, gene_name_list):
    added_name = []
    for each_ in _result:
        added_name.append([])
        for item in each_:
            # print(item, gene_name_list[item])
            added_name[-1].append(gene_name_list[item])
    return added_name


def get_index_partition(current_strand_list):
    forw_num = current_strand_list.count("+")
    rev_num = current_strand_list.count("-")
    if forw_num > rev_num:
        rev_index = []
        for i, j in enumerate(current_strand_list):
            if j == "-":
                rev_index.append(i)
        return rev_index
    elif rev_num > forw_num:
        forw_index = []
        for i, j in enumerate(current_strand_list):
            if j == "+":
                forw_index.append(i)
        return forw_index
    else:
        print("something wrong", current_strand_list)
        sys.exit(1)


def partition_s_rna(cluster_result, partition_index):
    for each_list in cluster_result:
        for each_item in each_list:
            if each_item in partition_index:
                # print("each:", each_)
                _index = cluster_result.index(each_list)
                each_list.remove(each_item)
                if cluster_result.count(each_list) > 1 or len(each_list) == 0:
                    cluster_result.remove(each_list)
                new_result = [each_item]
                if new_result not in cluster_result:
                    cluster_result.insert(_index, new_result)
    return cluster_result


def get_result_by_clustering(result_file, final_gene_strand, final_gene_index, final_gene_sort, matrix_i_j, threshold):
    threshold = threshold + 2
    result_list = []
    result_file_fp = open(result_file, 'w')
    i_iter = 0
    while i_iter < len(final_gene_index):
        # print(final_gene_index[i_iter])
        slice_start = final_gene_index[i_iter][0]
        slice_stop = final_gene_index[i_iter][-1]
        matrix_tmp = matrix_i_j[slice_start:slice_stop + 1, slice_start:slice_stop + 1]
        matrix_tmp = numpy.diag(matrix_tmp.diagonal(-1) + 2, -1)
        # print(matrix_tmp)
        gamma = Gamma(matrix_tmp)
        gamma.clustering()
        gamma_result = gamma.gamma_threshold(threshold)
        # print(result)
        current_gene_strand_list = final_gene_strand[i_iter]
        # print(current_gene_strand_list)
        tmp_index = get_index_partition(current_gene_strand_list)
        # print("result:", result)
        # print("tmp_index:", tmp_index)
        gamma_result = partition_s_rna(gamma_result, tmp_index)
        # print("final_result:", result)
        # print("\n")
        current_gene_name_list = final_gene_sort[i_iter]
        added_name_result = add_name(gamma_result, current_gene_name_list)
        for each_ in added_name_result:
            if each_ not in result_list:
                result_list.append(each_)
                result_file_fp.write(";".join(each_) + "\n")
        i_iter = i_iter + 1
    result_file_fp.close()


def get_result_by_clustering2(result_file, final_gene_strand, final_gene_index, final_gene_sort, matrix_i_j, threshold):
    threshold = threshold + 2
    result_list = []
    result_file_fp = open(result_file, 'w')
    i_iter = 0
    while i_iter < len(final_gene_index):
        # print(final_gene_index[i_iter])
        slice_start = final_gene_index[i_iter][0]
        slice_stop = final_gene_index[i_iter][-1]
        matrix_tmp = matrix_i_j[slice_start:slice_stop + 1, slice_start:slice_stop + 1]
        matrix_tmp = numpy.diag(matrix_tmp.diagonal(-1) + 2, -1)
        # print(matrix_tmp)
        gamma = Gamma(matrix_tmp)
        gamma_result = gamma.clustering2(threshold)
        current_gene_strand_list = final_gene_strand[i_iter]
        # print(current_gene_strand_list)
        tmp_index = get_index_partition(current_gene_strand_list)
        # print("result:", result)
        # print("tmp_index:", tmp_index)
        gamma_result = partition_s_rna(gamma_result, tmp_index)
        # print("final_result:", result)
        # print("\n")
        current_gene_name_list = final_gene_sort[i_iter]
        added_name_result = add_name(gamma_result, current_gene_name_list)
        for each_ in added_name_result:
            if each_ not in result_list:
                result_list.append(each_)
                result_file_fp.write(";".join(each_) + "\n")
        i_iter = i_iter + 1
    result_file_fp.close()



import time
import sys
import argparse

import os

from operondemmo.co_expression_matrix.c_i_j import compute_co_expression_by_c_i_j
from operondemmo.co_expression_matrix.person_i_j import compute_co_expression_by_person
from operondemmo.co_expression_matrix.spearman_i_j import compute_co_expression_by_spearman
from operondemmo.hierarchical_cluster.gamma_domain import get_result_by_clustering, get_result_by_clustering2
from operondemmo.input_file_handle.handle_gff import auto_download, generate_simple_gff, \
    get_gene_pos_strand, from_simple_gff_information_to_get, sorted_gene
from operondemmo.input_file_handle.handle_input import load_from_input_files, check_input_file, compute_expression
from operondemmo.input_file_handle.handle_kallisto import check_kallisto, split_from_input, generate_kallisto_index, \
    get_tpm_from_kallisto_quant, load_from_tpm_files
from operondemmo.version import version

self_version = version

APP_VERSION = (
        '''
    ----------------------------------------------------------------------
    
    operondemmo-(%s) - an independent demo of KNOWN operon predict method
    
    ----------------------------------------------------------------------
    ''' % self_version
)


def main():
    starting(prepare(sys.argv))


def prepare(argv):
    parser = argparse.ArgumentParser(description=APP_VERSION,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)

    advanced_argv = parser.add_argument_group("ADVANCED OPTIONS")
    parser.add_argument("-i", action="store", dest="input_dir", default="null",
                        help="A directory to store a group of files"
                             "default:need [samtools depth XXX > xxx.txt] result files;"
                             "when '--kallisto', need fastq files and fna file.")
    parser.add_argument("-o", action="store", dest="output_dir", default="OUT",
                        help="A directory include output data(operon file).default:OUT")
    parser.add_argument("-g", action="store", dest="gff_file", default="null",
                        help="The gff file of the prokaryote.")
    parser.add_argument("-p", action="store", dest="process_thread", default=1, type=int,
                        help="Specify the number of processing threads (CPUs).default:1")
    parser.add_argument("-t", action="store", dest="threshold", default=0.6, type=float,
                        help="the threshold in (-1,1)")
    advanced_argv.add_argument("-k", action="store", dest="kegg_id", default="null",
                               help="The kegg id of the prokaryote.(when '--auto_gff')")
    advanced_argv.add_argument("--auto_gff", action="store_true", dest="auto_gff", default=False,
                               help="Auto download gff_file from NCBI Database")
    advanced_argv.add_argument("--person", action="store_true", dest="person", default=False,
                               help="Build co-expression matrix with person correlation")
    advanced_argv.add_argument("--spearman", action="store_true", dest="spearman", default=False,
                               help="Build co-expression matrix with spearman correlation")
    advanced_argv.add_argument("--kallisto", action="store_true", dest="kallisto", default=False,
                               help="Build expression matrix with kallisto result")
    advanced_argv.add_argument("-v", "--version", action="version", version="operondemmo-" + self_version)
    if len(argv) == 1:
        print(parser.print_help())
        sys.exit(0)
    else:
        args = parser.parse_args(argv[1:])
        return args


def starting(args):
    """

    :type args: parser.parse_args()
    """
    if args.auto_gff:
        if args.kegg_id != "null":
            gff_file_path = auto_download(args.kegg_id)
        else:
            print("NEEDED KEGG ID.PLEASE check your input with option '-k'")
            return
    else:
        if args.gff_file != "null":
            gff_file_path = args.gff_file
        else:
            print("NEEDED GFF_FILE.PLEASE check your input with option '-g'")
            return
    if args.person:
        co_expression_method = 1
    else:
        if args.spearman:
            co_expression_method = 2
        else:
            co_expression_method = 0
    if args.input_dir[-1] != "/":
        input_dir = args.input_dir + "/"
    else:
        input_dir = args.input_dir
    if args.threshold > 1 or args.threshold < -1:
        print("IT CANNOT BE:", args.threshold, "PLEASE check your input with option '-t'")
        return
    if args.output_dir[-1] != "/":
        output_dir = args.output_dir + "/"
    else:
        output_dir = args.output_dir
    if not os.path.exists(output_dir):
        os.system("mkdir " + output_dir)
    operon_predict(args.threshold, input_dir, output_dir, gff_file_path,
                   co_expression_method, args.kallisto, args.process_thread)


def operon_predict(threshold, input_dir, output_dir, gff_file_path, co_expression_method, kallisto, p):
    # simple_gff_file_information
    print("from your gff file to get [gene_locus_tag, start, stop, strand]...")
    simple_gff_path = generate_simple_gff(gff_file_path, output_dir)
    gene_pos_dict, gene_strand_dict = get_gene_pos_strand(simple_gff_path)
    final_gene_strand, final_gene_index, final_gene_sort = \
        from_simple_gff_information_to_get(gene_pos_dict, gene_strand_dict)

    if kallisto:
        print("done\nRunning kallisto ...")
        print("from your fastq files to get tpm_co_expression_matrix...\n"
              "it would be cost few minutes, please waiting...")
        matrix_i_j = from_fastq_file_to_get_co_matrix_co_expression(input_dir, output_dir,
                                                                    gene_pos_dict, co_expression_method, p)
    else:
        print("done\nfrom your samtools_depth result files to get tpm_co_expression_matrix...\n"
              "it would be cost few minutes, please waiting...")
        # matrix_co_expression
        matrix_i_j = from_depth_file_to_get_co_matrix_co_expression(input_dir, gene_pos_dict, co_expression_method, p)
    # numpy.savetxt(output_path + "matrix.txt", matrix_i_j, fmt="%.8f")

    print("done\ngamma_domain clustering...")
    # hierarchical_cluster
    result_file = output_dir + "operon.txt"
    get_result_by_clustering2(result_file, final_gene_strand, final_gene_index, final_gene_sort, matrix_i_j, threshold)
    print("done")
    print("PLEASE open your output_path:", result_file)


def from_depth_file_to_get_co_matrix_co_expression(depth_files, gene_pos_dict, method, p):
    depth_files = load_from_input_files(depth_files)
    check_input_file(depth_files)

    gene_sort = sorted_gene(gene_pos_dict)

    matrix_groups_by_condition = compute_expression(depth_files, gene_pos_dict, gene_sort, p)
    matrix_co_expression = compute_co_expression(matrix_groups_by_condition, method)
    return matrix_co_expression


def from_fastq_file_to_get_co_matrix_co_expression(input_files, output_path, gene_pos_dict, method, p):
    check_kallisto()
    fna_file, fastq_files = split_from_input(input_files)
    check_input_file(fastq_files)
    gene_sort = sorted_gene(gene_pos_dict)
    kallisto_index = generate_kallisto_index(fna_file, gene_pos_dict, output_path, gene_sort)
    tpm_files = get_tpm_from_kallisto_quant(kallisto_index, fastq_files, output_path, p)
    tpm_matrix_by_condition = load_from_tpm_files(tpm_files)
    matrix_co_expression = compute_co_expression(tpm_matrix_by_condition, method)
    return matrix_co_expression


def compute_co_expression(expression_matrix, method):
    begin = time.time()
    if method == 0:
        matrix_c_i_j = compute_co_expression_by_c_i_j(expression_matrix)
        end = time.time()
        print("time: compute_co_expression_matrix,", end - begin)
        return matrix_c_i_j
    elif method == 1:
        matrix_c_person = compute_co_expression_by_person(expression_matrix)
        end = time.time()
        print("time: compute_co_expression_matrix,", end - begin)
        return matrix_c_person
    else:
        matrix_c_spearman = compute_co_expression_by_spearman(expression_matrix)
        end = time.time()
        print("time: compute_co_expression_matrix,", end - begin)
        return matrix_c_spearman


if __name__ == "__main__":
    main()



import sys
import os
from multiprocessing import Pool
import numpy
import time


def check_input_file(depth_files):
    num_depth_files = len(depth_files)
    if num_depth_files <= 1:
        print("need more condition")
        sys.exit(1)


def load_from_input_files(input_files):
    depth_files = []
    for each_file in os.listdir(input_files):
        depth_files.append(input_files + each_file)
    return depth_files


def read_depth_file(depth_file):
    file_content = open(depth_file, 'r').read().strip()
    content_list = file_content.split("\n")
    count_list = []
    for line in content_list:
        tmp_content = line.split("\t")
        count_list.append(tmp_content[-1])
    # print(len(count_list))
    return count_list


def compute_tpm(matrix_a, gene_pos_dict, gene_sort):
    count_matrix = numpy.array(matrix_a).astype('int').T
    # print(count_matrix.shape[0],count_matrix.shape[1])
    condition_num = count_matrix.shape[1]
    i = 0
    for item in gene_sort:
        sum_count = numpy.zeros([1, condition_num])
        len_gene = 0
        for (start, stop) in gene_pos_dict[item]:
            len_gene = len_gene + stop - start
            sum_count = sum_count + count_matrix[start - 1: stop, ...].sum(axis=0)
        average_count = sum_count / len_gene
        if i == 0:
            gene_count_matrix = average_count
        else:
            gene_count_matrix = numpy.row_stack((gene_count_matrix, average_count))
        i = i + 1
    sum_genes_matrix = gene_count_matrix.sum(axis=0)
    average_genes_matrix = gene_count_matrix / sum_genes_matrix
    return average_genes_matrix


def compute_average(matrix_a, gene_pos_dict, gene_sort):
    count_matrix = numpy.array(matrix_a).astype('int').T
    # print(count_matrix.shape[0],count_matrix.shape[1])
    condition_num = count_matrix.shape[1]
    i = 0
    for item in gene_sort:
        sum_count = numpy.zeros([1, condition_num])
        len_gene = 0
        for (start, stop) in gene_pos_dict[item]:
            len_gene = len_gene + stop - start
            sum_count = sum_count + count_matrix[start - 1: stop, ...].sum(axis=0)
        average_count = sum_count / len_gene
        if i == 0:
            gene_count_matrix = average_count
        else:
            gene_count_matrix = numpy.row_stack((gene_count_matrix, average_count))
        i = i + 1
    return gene_count_matrix


def compute_expression(depth_files, gene_pos_dict, gene_sort, p):
    start = time.time()
    pool = Pool(p)
    matrix_a = pool.map(read_depth_file, depth_files)
    pool.close()
    pool.join()
    # matrix_a = []
    # for each in depth_files:
    #     count_list = read_depth_file(each)
    #     matrix_a.append(count_list)
    matrix_tpm = compute_tpm(matrix_a, gene_pos_dict, gene_sort)
    end = time.time()
    print("compute tpm:", end - start)
    return matrix_tpm



from setuptools import setup, find_packages
from operondemmo.version import version
self_version = version

try:
    LONG_DESCRIPTION = open("README.rst", "rb").read().decode("utf-8")
except IOError:
    LONG_DESCRIPTION = "an independent demo of KNOWN operon predict method"

setup(
    name='operondemmo',
    version=self_version,
    keywords='operon',
    packages=find_packages(),
    url='https://github.com/GodInLove/operondemmo',
    license='GPLv3',
    author='yaodongliu',
    author_email='yd.liu.scu@gmail.com',
    description="an independent demo of KNOWN operon predict method",
    long_description=LONG_DESCRIPTION,
    install_requires=[
        'numpy>=1.14',
        'pandas',
    ],
    entry_points={
        "console_scripts": ['operondemmo = operondemmo.operon:main']
    },
)
